## convenience function for extracting coefficients and
## associated standard errors from fitted model object,
## potentially along with further information such as
## R^2 and error variance
gsummary <- function(x, ...) UseMethod("gsummary")

gsummary.lm <- function(x, digits = 3,
  R2 = TRUE, sigma = FALSE, sigma2 = TRUE, logLik = FALSE, ...)
{
  rval <- summary(x)
  rval <- cbind(rbind(coef(x), sqrt(diag(vcov(x)))),
    rbind(c(rval$r.squared, rval$sigma, rval$sigma^2, logLik(x)), NA))
  rownames(rval) <- rep("", nrow(rval))
  colnames(rval) <- c("(Intercept)", "value", "capital", "R^2", "sigma", "sigma^2", "logLik")
  print(round(rval[, c(1:3, if(R2) 4, if(sigma) 5, if(sigma2) 6, if(logLik) 7)],
    digits = digits), quote = FALSE, na.print = "")
  invisible(rval)
}

gsummary.systemfit <- function(x, digits = 3,
  R2 = NULL, sigma = NULL, sigma2 = NULL, ...)
{
  if(isTRUE(x$call$pooled)) {
    if(is.null(R2)) R2 <- FALSE
    if(is.null(sigma)) sigma <- FALSE
    if(is.null(sigma2)) sigma2 <- FALSE
    if(any(c(R2, sigma, sigma2)))
      stop("Currently, no additional information (R^2, sigma^2) available for pooled estimation.")
    
    rval <- summary(x)
    rval <- rbind(coef(x)[1:3], sqrt(diag(vcov(x)))[1:3])
    colnames(rval) <- c("(Intercept)", "value", "capital")
    rownames(rval) <- rep("", 2)
    
    cat("\nMethod: Pooled", x$method, "\n")
  } else {
    if(is.null(R2)) R2 <- TRUE
    if(is.null(sigma)) sigma <- FALSE
    if(is.null(sigma2)) sigma2 <- TRUE

    lab <- sapply(x$eq, function(z) z$eqnLabel)
    n <- length(lab)
  
    rval <- matrix(NA, nrow = n * 2, ncol = 6)
    rval[(1:n) * 2 - 1, 1:3] <- t(matrix(coef(x), nrow = 3))
    rval[(1:n) * 2,     1:3] <- t(matrix(sqrt(diag(vcov(x))), nrow = 3))
    rval[(1:n) * 2 - 1, 4]   <- sapply(x$eq, function(z) summary(z)$r.squared)
    rval[(1:n) * 2 - 1, 5]   <- sapply(x$eq, function(z) summary(z)$sigma)
    rval[(1:n) * 2 - 1, 6]   <- sapply(x$eq, function(z) summary(z)$sigma^2)

    colnames(rval) <- c("(Intercept)", "value", "capital", "R^2", "sigma", "sigma^2")
    rownames(rval) <- rep("", 2 * n)
    rownames(rval)[(1:n) * 2 - 1] <- lab

    cat("\nMethod:", x$method, "\n")
  }

  print(round(rval[, c(1:3, if(R2) 4, if(sigma) 5, if(sigma2) 6)],
    digits = digits), quote = FALSE, na.print = "")
  invisible(rval)
}

gsummary.plm <- function(x, digits = 3,
  R2 = TRUE, sigma = FALSE, sigma2 = TRUE, ...)
{
  nc <- length(coef(x))
  rss <- sum(residuals(x)^2)
  tss <- plm:::tss.plm(x)
  df <- df.residual(x)
  
  rval <- cbind(rbind(coef(x), sqrt(diag(vcov(x)))),
    rbind(c((tss - rss)/tss, sqrt(rss/df), rss/df), NA))
  rownames(rval) <- rep("", nrow(rval))
  colnames(rval)[nc + 1:3] <- c("R^2", "sigma", "sigma^2")
  print(round(rval[, c(1:nc, if(R2) nc+1, if(sigma) nc+2, if(sigma2) nc+3)],
    digits = digits), quote = FALSE, na.print = "")
  invisible(rval)
}
